package org.objectstyle.cayenne.dataview;

import java.util.Iterator;

import org.objectstyle.cayenne.access.DataContext;
import org.objectstyle.cayenne.conf.Configuration;
import org.objectstyle.cayenne.map.EntityResolver;

import junit.framework.TestCase;

public class DataViewTest extends TestCase {
	
	public void testManualCreate() throws Exception {
		DataView dataView = new DataView();
		
		DataContext dataContext = DataContext.createDataContext();

		dataView.setEntityResolver(dataContext.getEntityResolver());

		Configuration.getSharedConfiguration().loadDataView(dataView);
		
		checkDataView(dataView);
	}

	public void testCreateDataView() {
		DataView dataView = DataView.createDataView();
		
		checkDataView(dataView);
	}
	
	public void testThreadLocalDataView() {
		assertFalse(DataView.hasThreadDataView());

		DataView dataView = new DataView();
		
		assertFalse(DataView.hasThreadDataView());
		
		try {
			assertNull(DataView.getThreadDataView());
			assertTrue(false);
			
		} catch (IllegalStateException ise) {
			assertTrue(true);
		}
		
		DataView.bindThreadDataView(dataView);
		
		assertTrue(DataView.hasThreadDataView());
				
		assertEquals(dataView, DataView.getThreadDataView());
		
		DataView.bindThreadDataView(null);
		
		assertFalse(DataView.hasThreadDataView());
		
		try {
			assertNull(DataView.getThreadDataView());
			assertTrue(false);
			
		} catch (IllegalStateException ise) {
			assertTrue(true);
		}
	}
	
	private void checkDataView(DataView dataView) {
		ObjEntityView clientView = dataView.getObjEntityView("ClientView");
		
		assertEquals(1, dataView.getObjEntityViewNames().size());
		
		ObjEntityViewField oevf = clientView.getField("title");
		assertNotNull(oevf);
		assertEquals("title", oevf.getName());
		assertEquals("Title", oevf.getCaption());
		assertEquals(0, oevf.getIndex());
		assertEquals(0, oevf.getPreferredIndex());
		assertEquals(String.class, oevf.getJavaClass());

		oevf = clientView.getField("firstName");
		assertNotNull(oevf);
		assertEquals("firstName", oevf.getName());
		assertEquals("First Name", oevf.getCaption());
		assertEquals(1, oevf.getIndex());
		assertEquals(1, oevf.getPreferredIndex());
		assertEquals(String.class, oevf.getJavaClass());

		oevf = clientView.getField("lastName");
		assertNotNull(oevf);
		assertEquals("lastName", oevf.getName());
		assertEquals("Last Name", oevf.getCaption());
		assertEquals(2, oevf.getIndex());
		assertEquals(2, oevf.getPreferredIndex());
		assertEquals(String.class, oevf.getJavaClass());

		oevf = clientView.getField("active");
		assertNotNull(oevf);
		assertEquals("active", oevf.getName());
		assertEquals("Active", oevf.getCaption());
		assertEquals(3, oevf.getIndex());
		assertEquals(3, oevf.getPreferredIndex());
		assertEquals(Boolean.class, oevf.getJavaClass());

		oevf = clientView.getField("dateJoined");
		assertNotNull(oevf);
		assertEquals("dateJoined", oevf.getName());
		assertEquals("Date Joined", oevf.getCaption());
		assertEquals(4, oevf.getIndex());
		assertEquals(4, oevf.getPreferredIndex());
		assertEquals(java.util.Date.class, oevf.getJavaClass());

		oevf = clientView.getField("email");
		assertNotNull(oevf);
		assertEquals("email", oevf.getName());
		assertEquals("Email", oevf.getCaption());
		assertEquals(5, oevf.getIndex());
		assertEquals(5, oevf.getPreferredIndex());
		assertEquals(String.class, oevf.getJavaClass());
		
		for (Iterator i = clientView.getFields().iterator(); i.hasNext();) {
			 oevf = (ObjEntityViewField) i.next();
			 System.out.println("---------------");
			 System.out.println("Name:      " + oevf.getName());
			 System.out.println("Caption:   " + oevf.getCaption());
			 System.out.println("Index:     " + oevf.getIndex());
			 System.out.println("PreferredIndex:     " + oevf.getPreferredIndex());
			 System.out.println("JavaClass: " + oevf.getJavaClass());
			 System.out.println("DataType:  " + oevf.getDataType());
		}			
	}

}
